# Copyright(c) 2002 Association of Universities for Research in Astronomy, Inc.

procedure gemoffsetlist(infiles,reffile,tolerance,targetlist,offsetlist) 

# Version Aug 26, 2002  MT,JJ,IJ v1.4 release

char   infiles      {prompt="Input images"}
char   reffile      {prompt="Reference file"}
real   tolerance    {prompt="Maximum offset in arcsec or minutes from reference to offset positions"}
char   targetlist   {prompt="List of images at same position as reference"}
char   offsetlist   {prompt="List of images at different position from reference"}
char   offsettype   {"distance",prompt="Type of offset",enum="distance|time"}
char   key_xoff     {"XOFFSET",prompt="Header keyword for instrument x offset (in arcsec)"}
char   key_yoff     {"YOFFSET",prompt="Header keyword for instrument y offset (in arcsec)"}
char   key_time     {"UT",prompt="Header keyword for the time"}             
#bool   fl_addoffset {yes,prompt="Add offsets to key_xoff and key_yoff if not present?"}   
char   logfile      {"",prompt="Logfile"}
bool   verbose      {no,prompt="Verbose output?"}
int    status       {0,prompt="Exit status (0=good)"}
struct *scanfile    {"",prompt="Internal use only"} 

begin

char   l_infiles, l_logfile, l_reffile, l_targetlist, l_offsetlist
char   l_offsettype, l_key_time, l_key_xoff, l_key_yoff
bool   l_verbose, bad, fl_wcs #, needsfixing, l_fl_addoffset
real   l_tolerance
struct sdate
char   tmpinfiles, tmpokin, img, ref, tmpin, l_task
int    nbad, ii
real   xoff, yoff, xoffref, yoffref, x, y, r, l_pixscale

l_infiles=infiles; l_logfile=logfile; l_verbose = verbose
l_key_xoff=key_xoff; l_key_yoff=key_yoff
l_reffile=reffile; l_targetlist=targetlist; l_offsetlist=offsetlist
l_tolerance=tolerance; l_offsettype=offsettype; l_key_time=key_time
# l_fl_addoffset=fl_addoffset

# Start logging to file
if (l_logfile == "STDOUT") {
   l_logfile = ""
   l_verbose=yes
}
date | scan(sdate)
printlog("-------------------------------------------------------------\
-------------------",l_logfile,verbose=l_verbose)
printlog("GEMOFFSETLIST -- "//sdate,l_logfile,verbose=l_verbose)
printlog(" ",l_logfile,verbose=l_verbose)

# Initialize boolean flags
status = 0
fl_wcs = yes
bad = no

# Make temp files:
tmpin = mktemp("tmpin")
tmpinfiles = mktemp("tmpinfiles")
tmpokin =    mktemp("tmpokin")

# Check that target and offset lists do not exist
if(access(l_targetlist)) {
  printlog("ERROR - GEMOFFSETLIST: "//l_targetlist//" already exists.",l_logfile,verbose+)
  status = 1
}	
if(access(l_offsetlist)) {
  printlog("ERROR - GEMOFFSETLIST: "//l_offsetlist//" already exists.",l_logfile,verbose+)
  status = 1
}
if(status==1) goto clean

# Check that input files exist
status = 0
if(substr(l_infiles,1,1) =="@" && !access(substr(l_infiles,2,strlen(l_infiles)))) {
  printlog("ERROR - GEMOFFSETLIST: input file ("//substr(l_infiles,2,strlen(l_infiles))//") does not exist.",l_logfile,verbose+)
  status = 1
  goto clean
}	

delete(tmpinfiles,ver-, >>& "dev$null")
files(l_reffile//","//l_infiles,sort-, > tmpinfiles)

nbad=0
scanfile = tmpinfiles
while(fscan(scanfile,img) != EOF) {
  gimverify(img); img = gimverify.outname
  if(gimverify.status!=0) {
    printlog("WARNING - GEMOFFSETLIST: input image ("//img//") does not exist",l_logfile,verbose+)
    nbad+=1			
  } else print(img, >> tmpokin)			
} 
  
if(!access(tmpokin)) {
  printlog("ERROR - GEMOFFSETLIST: no input files.",l_logfile,verbose+)
  status = 1
  goto clean
}	

#------------------------------------------------------------
if(l_offsettype=="distance") {

# check to see that the right header keywords exist:
  scanfile = tmpokin
  nbad=0 
  fl_wcs=yes
  while(fscan(scanfile,img) != EOF) {
    keypar(img//"[0]","CRVAL1",silent+)
    if((!keypar.found) || (keypar.value=="INDEF") || (keypar.value=="0.")) {
      nbad+=1
    } else {
      keypar(img//"[0]","CRVAL2",silent+)
      if((!keypar.found) || (keypar.value=="INDEF") || (keypar.value=="0.")) nbad+=1
    }
# Figure out if some *APPWA* task has turned this into a spec WCS
     l_task=""
     fxhead(img,long+) | match("APPWA",stop-) | scan(l_task)
     if(l_task!="") {
        printlog("WARNING - GEMOFFSETLIST: Image "//img//" has been processed with "//l_task,l_logfile,verbose+)
        nbad+=1
     }
  }

  if(nbad!=0) {
    printlog("WARNING - GEMOFFSETLIST: "//(nbad-1)//" files do not have spatial WCS in the headers.",l_logfile,verbose+)
    printlog("                         Using instrument offsets instead.",l_logfile,verbose+)
    fl_wcs=no
    scanfile = tmpokin
    nbad=0
    while(fscan(scanfile,img) != EOF) {
      keypar(img//"[0]",l_key_xoff,silent+)
      if((!keypar.found) || (keypar.value=="INDEF")) {
        nbad+=1
      } else {
        keypar(img//"[0]",l_key_yoff,silent+)
        if((!keypar.found) || (keypar.value=="INDEF")) nbad+=1
      } 
    }

    if(nbad!=0) {
      printlog("ERROR - GEMOFFSETLIST: "//(nbad-1)//" files do not have instrument offset information.",l_logfile,verbose+)
      status = 1
      goto clean
    }
  }

# Get the distance offsets from the header
  delete(tmpinfiles,ver-, >>& "dev$null")	
  scanfile = tmpokin
  while(fscan(scanfile,img) != EOF) {
  nbad=0
    if(!fl_wcs) {
      keypar(img//"[0]",l_key_xoff,silent+); xoff = real(keypar.value)
      keypar(img//"[0]",l_key_yoff,silent+); yoff = real(keypar.value)
     print(img//" "//xoff//" "//yoff, >> tmpinfiles)
    } else {
      print(img//"[1]", >> tmpin)
    }
  }

# Get the WCS shifts
  if(fl_wcs) {
     head(tmpin,nlines=1) | scan(ref)
      wcsmap("@"//tmpin,ref,"dev$null",fitgeom="shift",interactive-) | \
      match("shift",stop-) | fields("STDIN","5,6") | join(tmpokin,"STDIN", > tmpinfiles)
     delete(tmpin,ver-)
# Get the pixel scale and convert to arcsec
     head(tmpinfiles,nlines=1) | scan(ref)
     keypar(ref//"[0]","PIXSCALE",silent+)
     if(keypar.found) {
         l_pixscale=real(keypar.value)
     } else {
        keypar(ref//"[0]","CD1_1",silent+)
         l_pixscale=real(keypar.value)
        keypar(ref//"[0]","CD1_2",silent+)
         l_pixscale=sqrt(l_pixscale**2+real(keypar.value)*real(keypar.value))*3600.
     }
     tcalc(tmpinfiles,"c2","-c2*"//str(l_pixscale))
     tcalc(tmpinfiles,"c3","-c3*"//str(l_pixscale))
  } 

} else {
# Get the time from the header
  scanfile = tmpokin
  delete(tmpinfiles,ver-, >>& "dev$null")	
  nbad=0
  while(fscan(scanfile,img) != EOF) {
    keypar(img//"[0]",l_key_time,silent+); xoff = real(keypar.value)
    if(!keypar.found) {
      nbad+=1
    } else {
      print(img//" "//xoff, >> tmpinfiles)	
    }
  }	
  if(nbad!=0) {
    printlog("ERROR - GEMOFFSETLIST: "//nbad//" files do not have time in header.",l_logfile,verbose+)
    status = 1
    goto clean
  }	
}

#-----------------------------------------------------------------
# Get the info from the header of the reference frame
if(l_offsettype=="distance") {
  head(tmpinfiles,nlines=1) | scan(ref,xoffref,yoffref)
} else {     
  head(tmpinfiles,nlines=1) | scan(ref,xoffref)
}
# Group files in tmpinfile

if(l_offsettype=="distance") {
  scanfile = tmpinfiles
  ii=0
  while(fscan(scanfile,img,xoff,yoff) != EOF) {
    ii+=1
    if(ii>1) {
        x = xoff - xoffref
        y = yoff - yoffref
      r = sqrt(x*x+y*y)
      if(r<l_tolerance) {
        print(img, >> l_targetlist)
      } else {
        print(img, >> l_offsetlist)
      }	
      x = (int(x*100))/100.
      y = (int(y*100))/100.
      if(l_key_xoff!="GXOFF" && l_key_yoff!="GYOFF") {
        gemhedit(img//"[0]","GOFFREF",l_reffile,"GEMOFFSETLIST spatial reference image")
        gemhedit(img//"[0]","GXOFF",x,"x offset from reference image (arcsec)")
        gemhedit(img//"[0]","GYOFF",y,"y offset from reference image (arcsec)")
        hedit(img//"[0]","GTOFF","",up+,ad-,del+,ver-,show-)
      }
    }
  }	
} 
# Don't update headers if not distance offset
 else {
  scanfile = tmpinfiles
  ii=0
  while(fscan(scanfile,img,xoff) != EOF) {
    ii+=1
    if(ii>1) {
      x = abs(xoff-xoffref)*60.
      if(x<l_tolerance) { 
        print(img, >> l_targetlist)
      } else { 	
        print(img, >> l_offsetlist)
      }
      x = (int(x*1000))/1000.
##      gemhedit(img//"[0]","GOFFREF",l_reffile,"GEMOFFSETLIST reference image")
##      gemhedit(img//"[0]","GTOFF",x,"Time offset from reference image (min)")
##      hedit(img//"[0]","GXOFF","",up+,ad-,del+,ver-,show-)
##      hedit(img//"[0]","GYOFF","",up+,ad-,del+,ver-,show-)
    }
  }
}

clean:
   scanfile = ""
   delete(tmpinfiles//","//tmpokin,verify-, >>& "dev$null")
   if(status==0) {
     printlog("GEMOFFSETLIST: Exit status good.",l_logfile,l_verbose)
     printlog("--------------------------------------------------------------------------------",l_logfile,verbose=l_verbose)	
   }

end
