# GMOS example reductions script: Typical reduction of imaging data
#
# This data processing was done to aid the data quality assessment performed
# by the Gemini staff. The data processing is not designed to give the best
# possible result. Better signal-to-noise and better cleaning for cosmic-ray
# hits and bad pixels will most likely be possible. The user of these data
# is encouraged to use the provided co-added images only as guide lines and
# to re-reduce the data to obtain the best possible reduction.
#
# Gemini GMOS data reduction script
# Observation UT date:  2002jun06
# Data processor:  Inger Jorgensen
# Data reduction date:  2002jun07
# Revisions: 2002aug26 garith changed to gemarith
#
# Brief data description: Imaging for the SV program GN-2002A-SV-78

gmos.logfile="20020606_GN-2002A-SV-78.log"

delete glist ver-
for(i=140; i<=142; i+=1) 
 print("N20020606S0000"+i, >> "glist")

gireduce @glist bias=../../2002may16/Basecalib/N20020507S0045_bias \
flat1=../../2002jun14/Basecalib/N20020606S0149_flat \
rawpath="gdata$2002jun06/"

delete rglist ver-
sections rg//@glist > rglist
gmosaic @rglist 

delete mrglist ver-
sections m//@rglist > mrglist

# The following gives an example of how to make a fringe frame for GMOS
# The NIRI task nisky is used, thus the user needs to load the NIRI package
#
# Make a fringe frame - not ideal since there are only 3 images
# subtract off median sky
gemarith mrgN20020606S0140 - 14852.0 smrgN20020606S0140 logfile="20020606_GN-2002A-SV-78.log"
gemarith mrgN20020606S0141 - 14556.0 smrgN20020606S0141 logfile="20020606_GN-2002A-SV-78.log"
gemarith mrgN20020606S0142 - 13831.0 smrgN20020606S0142 logfile="20020606_GN-2002A-SV-78.log"
delete smrglist ver-
sections sm//@rglist > smrglist
# Fool nisky, which looks for NPREPARE and SATURATI in the image headers
hedit mr*fits[0] "NPREPARE" "fake" add+ ver- upd+
hedit mr*fits[0] "SATURATI" "200000" add+ ver- upd+

# Use nisky to make the mask files
nisky @mrglist fl_keep+ fwhm=6 logfile="20020606_GN-2002A-SV-78.log" \
combtype=median fl_vardq- maskfactor=0.5
hedit smrgN20020606S0140[1] BPM mrgN20020606S0140msk.pl add+ upd+ ver-
hedit smrgN20020606S0141[1] BPM mrgN20020606S0141msk.pl add+ upd+ ver-
hedit smrgN20020606S0142[1] BPM mrgN20020606S0142msk.pl add+ upd+ ver-

# Then use gemcombine to combine the sky subtracted individual images
gemcombine @smrglist fringe combine=median reject=avsigclip \
lsigma=3 hsigma=3 grow=1 logfile="20020606_GN-2002A-SV-78.log"

# A bit more rough cleaning to avoid some left over object signal
imreplace fringe[1] 0 lower=200
# Smooth this since the S/N is quite low
cp fringe.fits mfringe.fits
median mfringe[1] mfringe[1,overwrite] 7 7 
imreplace  mfringe[1][2055:2305,1580:1890] 0 lower=50

# There will still be some artifacts introduced by the fringe frame
# but it should be better than no fringe correction

# Subtract off the fringe frame
gemarith mrgN20020606S0140 - mfringe fmrgN20020606S0140 logfile="20020606_GN-2002A-SV-78.log"
gemarith mrgN20020606S0141 - mfringe fmrgN20020606S0141 logfile="20020606_GN-2002A-SV-78.log"
gemarith mrgN20020606S0142 - mfringe fmrgN20020606S0142 logfile="20020606_GN-2002A-SV-78.log"

delete fmrglist ver-
sections fm//@rglist > fmrglist
imcoadd @fmrglist badpixf="gmos$data/mgmos_bpm22.pl" \
fwhm=9 threshold=100 logfile="20020606_GN-2002A-SV-78.log" 

gemseeing fmrgN20020606S0140_add fl_inter+ logfile=20020606_GN-2002A-SV-78.log fl_over+
gemqa  fmrgN20020606S0140_add pass pass pass pass logfile=20020606_GN-2002A-SV-78.log

